<?php
namespace Teknisa\CMP\Modules\Solicitacao\Modules\Consolidacao\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\Report\ReportService;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use Teknisa\CMP\Modules\Solicitacao\Modules\Consolidacao\Services\sup00100ConsolidacaoDeSolicitacaoService;

class sup00100ConsolidacaoDeSolicitacaoController {
  
	protected $connection;
	protected $newCode;
  
    public function __construct(sup00100ConsolidacaoDeSolicitacaoService $service, $entityManager, $environment, $newCode) {
        $this->service      = $service;
        $this->connection   = $entityManager->getConnection();
        $this->environment  = $environment;
        $this->newCode      = $newCode;
    }
    
    public function validaFilial(DTO\Request\Row $request, DTO\Response $response)
    {
	    $row     = $request->getRow();
		$message = '';
		
		$filialPrinc   = self::validaUnidade($row['CDFILIALPRINC'], $row['CONSIDERAPAR'], $row);
		
		if ($filialPrinc === true && $row['CONSIDERAEST'] == 'S') {
			$filialEst = self::validaUnidade($row['CDFILIALEST'], $row['CONSIDERAPAR'], $row);
		} else {
		    $message = $filialPrinc;
		    if ($row['CONSIDERAEST'] == 'S'){
		        $filialEst = false;
		    } else {
		        $filialEst = true;
		    }
		}
		
		
		if(!is_bool($filialEst)){
		    $message = $filialEst;
		}
		
		if ($filialPrinc === true && $filialEst === true) {
 			if (is_array($row['CHAVE']) && !empty($row['CHAVE'])) {
				$NRLISTA = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrorg());
				
				foreach ($row['CHAVE'] as $value) {
				    $this->service->insLista($NRLISTA, $value);
				}
			} else {
 			    $NRLISTA = null;
 			}
 			
			$CDFILCENT = $this->service->queryCentComp($row['CDFILIAL']);
			$respStPc = self::accessProcedure($row, $NRLISTA, $CDFILCENT[0]['CDFILCENTCMP']);
			
			if(empty($respStPc) || $respStPc == null){
				//mensagem de Erro - Não há dados para a consolidação de solicitações.
				$message = 'Não há dados para a consolidação de solicitações.';
			}else{
                $message = 'Solicitação consolidada foi gerada com o número '. $respStPc;			    
			}
		}
		$response->addMessage(new Message($message));
    }
    
    public function validaUnidade($filial, $paramOl, $row){
    	$validaFilial = $this->service->validaFilial($filial);
    	$msg;
    	if(empty($validaFilial)){
	        //mensagem de Erro - 
	        $msg = 'Unidade não cadastrada.';
	        //$response->addMessage(new Message('Unidade não cadastrada.'));
	        return $msg;
	    }else{
    	    if($validaFilial[0]['IDCNTRRESC'] == 'S'){
    	        //mensagem de Erro - 
    	        $msg = 'Unidade com contrato rescindido.';
    	        //$response->addMessage(new Message('Unidade com contrato rescindido.'));
    	        return $msg;
    	    }
    	    if(empty($validaFilial[0]['CDREGIAO'])){
    	        //mensagem de Erro - 
    	        $msg = 'Não há região cadastrada para unidade informada.';
    	        //$response->addMessage(new Message('Não há região cadastrada para unidade informada.'));
    	        return $msg;
    	    }
    	    $queryParaCmpSup = $this->service->queryParaCmpSup($row);
    	    if(empty($queryParaCmpSup)){
    	        //mensagem de Erro - 
    	        $msg = 'Não  há parâmetros de compras para a unidade.';
    	        //$response->addMessage(new Message('Não  há parâmetros de compras para a unidade.'));
    	        return $msg;
    	    }
    	    if($paramOl == 'S' && $validaFilial[0]['IDFUNCAOFILI'] != 'O'){
    	    	//mensagem de Erro - 
    	    	$msg = 'Unidade posicionada deve ser uma Unidade OL para considerar parametrizaçao OL.';
    	   // 	$response->addMessage(new Message('Unidade posicionada deve ser uma Unidade OL para considerar parametrizaçao OL.'));
    	    	return $msg;
    	    }
	    }
	    return true;
    }
    
    public function dateFormat($data){
        $data = explode("/", $data);
        $data = $data[2] . "-" . $data[1] . "-" . $data[0];
        $data = date_create($data);
        $data = $data->format('d/m/y');
        return $data;
    }
    
    public function accessProcedure($row, $NRLISTA, $CDFILCENT){
    	$connection = $this->connection;
    	$CDCOMPRADOR = is_array($row['CDOPERADOR'])  ? !empty($row['CDOPERADOR'])  ? self::concatArray($row['CDOPERADOR'])  : NULL : NULL;
    	$CDFILIAL    = is_array($row['CDFILIALSOL']) ? !empty($row['CDFILIALSOL']) ? self::concatArray($row['CDFILIALSOL']) : NULL : NULL;
        
        //formatar datas
        $row['DTPROCESS'] = self::dateFormat($row['DTPROCESS']);
        
        $procedure = new StoredProcedure($connection, 'CONSOLIDA_SOLICITA');
        $procedure->addParam(new Param('P_CDFILCONS',	 Param::PARAM_INPUT, $row['CDFILIALPRINC'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILCENT',    Param::PARAM_INPUT, $CDFILCENT,            Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT, $CDFILIAL,             Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILEST',     Param::PARAM_INPUT, $row['CDFILIALEST'],   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCOMPRADOR',  Param::PARAM_INPUT, $CDCOMPRADOR,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTENTRINI',    Param::PARAM_INPUT, $row['DATAINI'],       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTENTRFIN',    Param::PARAM_INPUT, $row['DATAFIN'],       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDPRODINI',    Param::PARAM_INPUT, $row['CDPRODINI'],     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDPRODFIN',    Param::PARAM_INPUT, $row['CDPRODFIN'],     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDCONSESTOQ',  Param::PARAM_INPUT, $row['CONSIDERAEST'],  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDCONSOLIDA',  Param::PARAM_INPUT, $row['CONSOLIDA'],     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDPRODORIG',   Param::PARAM_INPUT, $row['CONSIDERAPROD'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTPROCESS',    Param::PARAM_INPUT, $row['DTPROCESS'],     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_LISTA',   	 Param::PARAM_INPUT, $NRLISTA,              Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CONSSOL', 	 Param::PARAM_INPUT, $row['CONSISOL'],      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CONSAF',  	 Param::PARAM_INPUT, $row['CONSIAF'],       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CONSLIMSEG',   Param::PARAM_INPUT, $row['CONSIDERALIM'],  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CONSOL',  	 Param::PARAM_INPUT, $row['CONSIDERAPAR'],  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRSOLICMP',    Param::PARAM_OUTPUT, NULL,                 Param::PARAM_TYPE_STR, 20));
        $rowsaffect = $procedure->execute();
        return $rowsaffect["P_NRSOLICMP"];
    }
    
    public function concatArray($filialArray){
        $filiais = implode("_", $filialArray);
        return $filiais;
    }
}
	        //$response->addDataSet(new DataSet('res', $validacao));